/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: error.c,v $
   Revision 1.2  2000/07/04 12:54:35  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:44  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* System headers */

#ifdef _REENTRANT
#include <pthread.h>
#endif /* _REENTRANT */

/* Local headers */

#include "pingutil.h"

/* File scope variables */

/* this stuff is not MT-Safe */

static uint_t __P_error_serial = 0;
static char *__P_error_progname = "<program>";

#ifdef _REENTRANT

static pthread_once_t __P_error_once = PTHREAD_ONCE_INIT;
static pthread_key_t __P_error_key;

#else

int p_errno = 0; /* global PING error number */

#endif /* _REENTRANT */

/* Functions */

void P_error_init(const char *progname)
  {
  char *p, *q;

  if(progname)
    __P_error_progname = (char *)progname;
  }

/*
 */

void P_error_printf(const char *fmt, ...)
  {
  va_list vp;

  fprintf(stderr, "%s(%i): ", __P_error_progname, __P_error_serial++);
  va_start(vp, fmt);
  vfprintf(stderr, fmt, vp);
  va_end(vp);
  fflush(stderr);
  }

/*
 */

void P_error_syserr(void)
  {

  fprintf(stderr, "%s(%i): %s\n", __P_error_progname, __P_error_serial++,
	  strerror(errno));
  fflush(stderr);
  }

/*
 */

#ifdef _REENTRANT

static void __P_error_init_once(void)
  {
  int *e = P_new(int);

  *e = 0;
  pthread_key_create(&__P_error_key, NULL);
  pthread_setspecific(__P_error_key, (void *)e);
  }

/*
 */

int P_error_get_errno(void)
  {
  int *e;
  
  pthread_once(&__P_error_once, __P_error_init_once);

  e = (int *)pthread_getspecific(__P_error_key);
  return(*e);
  }

/*
 */

void P_error_set_errno(int err)
  {
  int *e;
  
  pthread_once(&__P_error_once, __P_error_init_once);

  e = (int *)pthread_getspecific(__P_error_key);
  *e = err;
  }

#endif /* _REENTRANT */

/* end of source file */
