/* ----------------------------------------------------------------------------
   Ping Utility Library
   Copyright (C) 1994-2000 PING Software Group and Mark A Lindner

   This file is part of pingutil.
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   ----------------------------------------------------------------------------
   $Log: exec.c,v $
   Revision 1.2  2000/07/04 12:54:36  markl
   More fixes.

   Revision 1.1  2000/06/14 06:02:44  ping
   Initial checkin.

   ----------------------------------------------------------------------------
*/

/* Feature test switches */

#include "config.h"

/* Feature test switches */

#define _POSIX_C_SOURCE 199506L

/* System headers */

#include <sys/wait.h>
#include <sys/times.h>
#include <fcntl.h>

/* Local headers */

#include "pingutil.h"

/* Macros */

#define P_EXEC_NULLDEV "/dev/null"

/* Functions */

int P_exec_run(char **argv, int fdin, int fdout, bool_t waitf)
  {
  pid_t pid;
  int sd, fdz = -1, r = 0;

  pid = fork();
  switch(pid)
    {
    case -1:
      perror("fork");
      exit(EXIT_FAILURE);

    case 0: /* child process */
      if(fdin < 0 || fdout < 0)
        fdz = open(P_EXEC_NULLDEV, O_RDWR);
      dup2(((fdin < 0) ? fdz : fdin), STDIN_FILENO);
      dup2(((fdout < 0) ? fdz : fdout), STDOUT_FILENO);
      dup2(((fdout < 0) ? fdz : fdout), STDERR_FILENO);
      execvp(*argv, argv);
      if(fdz >= 0) close(fdz);
      perror("execvp");
      exit(EXIT_FAILURE);

    default: /* parent process */
      if(waitf)
        r = P_exec_wait(pid);
      return(r);
    }
  }

/*
 */

int P_exec_pipefrom(char **argv, int *fd)
  {
  int pfd[2];

  if(pipe(pfd))
    {
    perror("pipe");
    exit(EXIT_FAILURE);
    }
  *fd = pfd[0];
  fcntl(pfd[0], F_SETFD, FD_CLOEXEC);

  return(P_exec_run(argv, -1, pfd[1], FALSE));
  }

/*
 */

int P_exec_pipeto(char **argv, int *fd)
  {
  int pfd[2];

  if(pipe(pfd))
    {
    perror("pipe");
    exit(EXIT_FAILURE);
    }
  *fd = pfd[1];
  fcntl(pfd[1], F_SETFD, FD_CLOEXEC);

  return(P_exec_run(argv, pfd[0], -1, FALSE));
  }

/*
 */

int P_exec_wait(pid_t pid)
  {
  int st = 0;

  if(pid)
    waitpid(pid, &st, 0);
  else
    wait(&st);

  return(WEXITSTATUS(st));
  }

/*
 */

int P_exec_va_run(int fdin, int fdout, bool_t waitf, ... /* , NULL */)
  {
  char **v;
  va_list vp;
  int r;

  va_start(vp, waitf);
  v = P_string_valist2vec(NULL, vp, NULL);
  va_end(vp);

  r = (v ? P_exec_run(v, fdin, fdout, waitf) : -1);
  if(v)
    P_free(v);

  return(r);
  }

/*
 */

int P_exec_va_call(const char *arg, ... /* , NULL */)
  {
  char **v;
  va_list vp;
  int r;

  va_start(vp, arg);
  v = P_string_valist2vec(arg, vp, NULL);
  va_end(vp);

  r = (v ? P_exec_run(v, STDIN_FILENO, STDOUT_FILENO, TRUE) : -1);
  if(v)
    P_free(v);

  return(r);
  }

/* end of source file */
